<?php
namespace Util;

/**
 * Class Utilities
 *
 * @package Util
 */

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Teknisa\Libs\Util\Environment;
use Util\Instance\Repository;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\HTTP\UploadedFile;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\DevOps\Util\Utilities AS DevOpsUtilities;
use Util\DevOpsEnvironment AS DevOpsEnvironment;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use MongoDB\BSON\UTCDateTime as MongoDate;

class Utilities {

    /** @var  \DateTime(); */
    protected $currentTime;
    /** @var  Session */
    protected $session;
    /** @var \Teknisa\Libs\Util\Environment */
    private $environment;
    /** @var DevOpsEnvironment */
    private $devOpsEnvironment;

    /**
     * @var PmEnvironment
     */
    private $pmEnvironment;
    const LINE_BREAK = "
";

    public function __construct(Session $session,Environment $environment, DevOpsEnvironment $devOpsEnvironment,
                                PmEnvironment $pmEnvironment) {
        $this->session = $session;
        $this->currentTime = new \DateTime();
        $this->environment = $environment;
        $this->devOpsEnvironment = $devOpsEnvironment;
        $this->pmEnvironment = $pmEnvironment;
    }

    public function getSession() {
        return $this->session;
    }

    public function setInSession($key, $value) {
        $this->session->set($key, $value);
    }

    public function hasInSession($key) {
        return $this->session->has($key);
    }
    public static function getVersionDelphiMin(){
        return InstanceManager::getInstance()->getParameter('versionDelphiMin');
    }
    public static function getUrlWSDLSecurityTeknisa(){
        return InstanceManager::getInstance()->getParameter('PER_URL');
    }
    public static function getUrlWSDLPatchTeknisa(){
        return InstanceManager::getInstance()->getParameter('PATCH_URL');
    }
    public static function getTicketUrl(){
        return InstanceManager::getInstance()->getParameter('TICKET_URL');
    }
    public static function getErpUrl(){
        return InstanceManager::getInstance()->getParameter('ERP_FIN_URL');
    }
    public static function getReportPebbianUrl(){
        return InstanceManager::getInstance()->getParameter('REPORT_PEBBIAN_URL');
    }
    public static function getPmUrl(){
        return InstanceManager::getInstance()->getParameter('PM_URL');
    }
    public static function getWaitingTicket(){
        return InstanceManager::getInstance()->getParameter('WAITING_TICKET');
    }
    public static function getActiveTicket(){
        return InstanceManager::getInstance()->getParameter('activeTicket');
    }
    public static function getSupportOperatorPasswordExpiryTime()
    {
        return InstanceManager::getInstance()->getParameter('supportOperatorPasswordExpiryTime');
    }
    public function getInSession($key) {
        return $this->session->get($key);
    }

    public function removeFromSession($key) {
        $this->session->remove($key);
    }

    public static function getTypeByExtension($key) {
        $array = array(
            "js" => "application/javascript",
            "json" => "application/json",
            "xml" => "application/xml",
            "zip" => "application/zip",
            "pdf" => "application/pdf",
            "sql" => "application/sql",
            "doc" => "application/msword",
            "docx" => "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
            "xls" => "application/vnd.ms-excel",
            "xlsx" => "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            "ppt" => "application/vnd.ms-powerpoint",
            "pptx" => "application/vnd.openxmlformats-officedocument.presentationml.presentation",
            "odt" => "application/vnd.oasis.opendocument.text",
            "css" => "text/css",
            "html" => "text/html",
            "csv" => "text/csv",
            "plain" => "text/plain",
            "png" => "image/png",
            "jpeg" => "image/jpeg",
            "gif" => "image/gif"
        );
        $type = null;
        if(isset($array[$key])){
            $type = $array[$key];
        }
        return $type;
    }

    public function getCurrentTime() {
        return clone $this->currentTime;
    }
    public static function getCurrentTimeStatic() {
        return clone self::getEnvironment()->getCurrentTime();
    }

    public function getCurrentUserId() {
        return $this->environment->getCurrentUserId();
    }

    public function getCurrentOrganizationId() {
       return $this->environment->getCurrentOrganizationId();
    }

    public function getPmCurrentOrganizationId() {
       return $this->environment->getCurrentOrganizationId();
    }

    public function getPmCurrentUserId()
    {
        return $this->pmEnvironment->getCurrentUserId();
    }
    public static function getTemporaryDir()
    {
        return InstanceManager::getInstance()->getParameter('temporaryDir');
    }
    public static function getValidateBudgetFaseGep()
    {
        return InstanceManager::getInstance()->getParameter('validateBudgetFaseGep');
    }
    public static function getRefuseBudgetFaseGep()
    {
        return InstanceManager::getInstance()->getParameter('refuseBudgetFaseGep');
    }
    public static function getExcludedWaitingCustomerLogFaseTram()
    {
        return InstanceManager::getInstance()->getParameter('excludedWaitingCustomerLogFaseTram');
    }

    public function setDoCurrentUserId($userId)
    {
        $this->devOpsEnvironment->setCurrentUserId($userId);
    }
    public function setDoOrganizationId($organizationId)
    {
        $this->devOpsEnvironment->setOrganizationId($organizationId);
    }

    public static function onlyNumber($str) {
        return preg_replace("/[^0-9]/", "", $str);
    }

    public function setDOCurrentUserIdByCurrentPMUser()
    {
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        $PMCurrentUser = $this->getPMCurrentUser();
        /** @var  \Zeedhi\Model\Profile\Entities\ZhUser $DOCurrentUser */
        $DOCurrentUser = $entityManager->getRepository(DevOpsUtilities::PROFILE_ENTITY_DIR . 'ZhUser')
            ->findOneBy(array('externalResId'=>$PMCurrentUser->getId()));
        $this->devOpsEnvironment->setCurrentUserId($DOCurrentUser->getId());
        $this->devOpsEnvironment->setOrganizationId($DOCurrentUser->getOrganization()->getId());
    }

    public static function getCloseIssueTaskStatus(){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskProperties $taskProperties */
        $taskProperties = self::getEntityManager()->getRepository(DevOpsUtilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'TaskProperties')
            ->findOneBy(array('organization' => self::getEnvironment()->getPmCurrentOrganizationId()));
        return $taskProperties->getFinalTaskStatus();
    }
    public static function getBacklogIssueTaskStatus(){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskProperties $taskProperties */
        return self::getEntityManager()->getRepository(DevOpsUtilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('organization' => self::getEnvironment()->getPmCurrentOrganizationId(),'CONTROL' => 'B'));
    }

    public function setPMCurrentUserId($userId)
    {
        $this->pmEnvironment->setCurrentUserId($userId);
        $this->setDOCurrentUserIdByCurrentPMUser();
    }

    /**
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource
     */
    public function getPMCurrentUser()
    {
        $entityManager = self::getEntityManager();
        return $entityManager->find(DevOpsUtilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Resource', $this->getPmCurrentUserId());
    }

    public function setOrganizationId($orgId)
    {
        $this->session->set('organizationId', $orgId);
    }

    public function setCdClienteCurrent($cdCiente)
    {
        $this->session->set('cdCiente', $cdCiente);
    }

    public function getCurrentCdCliente()
    {
        return $this->session->has("cdCiente") ? $this->session->get("cdCiente") : null;
    }

    public static function getDataRequestFilter($filter = array(), $itemsPerPage = 500, $page = 1){
        $dataFilter = array(
            "requestType" => \Zeedhi\Framework\DTO\Request::TYPE_FILTER,
            "filter" => $filter,
            "itemsPerPage" => $itemsPerPage,
            "page" => $page
        );

        return $dataFilter;
    }

    public function getToken()
    {
        return $this->environment->getLoginTokenInSession();
    }

    public function getCurrentDevOpsToken() {
        return $this->session->has("devOpsToken") ? $this->session->get("devOpsToken") : null;
    }

    public function setCurrentDevOpsToken($token) {
        $this->session->set('devOpsToken', $token);
    }

    /**
     * @return \Teknisa\Model\Geral\Entities\Operador
     */
    public function getCurrentUser() {
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador')->find(
            array(
                'cdoperador' => $this->getCurrentUserId(),
                'nrorg' => $this->getCurrentOrganizationId(),
            )
        );
    }

    /**
     * @return \Teknisa\Model\Geral\Entities\Cliente
     */
    public function getCurrentCustomer() {
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository('\Teknisa\Model\Geral\Entities\Cliente')->find(
            array(
                'cdcliente' => $this->getCurrentCdCliente(),
                'nrorg' => $this->getCurrentOrganizationId(),
            )
        );
    }

    /**
     * Turns an array of associative arrays into an associative array of associative arrays that has desired key property
     * as keys and associative array that has that key as value
     * @param array $array array of associative arrays
     * @param string $id desired key property
     * @return array associative array (desiredKey => respectiveAssociativeArrayAsValue)
     */
    public static function objectsByID(array $array, $id = 'ID') {
        $newArray = array();
        foreach($array as $element) {
            $newArray[$element[$id]] = $element;
        }
        return $newArray;
    }

    /**
     * @param String $email
     * @return int
     */
    public static function validateEmail($email)
    {
        $emailRegex = "/^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/";
        return preg_match($emailRegex, $email);
    }

    public static function percent($value,$total)
    {
        if($total){
            $percent = round(100 - ($value / $total) * 100, 1);
        }else{
            $percent = 0;
        }

        return $percent;
    }

    public static function validateNameLogin($name)
    {
        $nameRegex = "/^[a-zA-ZàáâäãåąčćęèéêëėįìíîïłńòóôöõøùúûüųūÿýżźñçčšžÀÁÂÄÃÅĄĆČĖĘÈÉÊËÌÍÎÏĮŁŃÒÓÔÖÕØÙÚÛÜŲŪŸÝŻŹÑßÇŒÆČŠŽ∂ð ,.'-]+$/";
        return preg_match($nameRegex, $name);
    }

    /**
     * Receive two dates in string an format [default = d/m/Y] and return
     *  1 if $date1 is after  $date2
     *  0 if $date1 is equal  $date2
     * -1 if $date1 is before $date2
     * @param string $date1
     * @param string $date2
     * @param string $format
     * @return int
     */
    public static function compareDates($date1, $date2, $format = 'd/m/Y')
    {
        $date1 = \DateTime::createFromFormat($format, $date1);
        $date2 = \DateTime::createFromFormat($format, $date2);
        $return = intval($date1->diff($date2)->format('%r%s'));
        return $return === 0 ? 0 : $return > 0 ? -1 : 1;
    }

    public static function zeroLeftPadding($str, $pad = 12) {
        return str_pad($str, $pad, '0', STR_PAD_LEFT);
    }

    private static function buildBetweenDatesCriteria($values) {
        $operator = 'BETWEEN';
        $dateFormat = 'DD/MM/YYYY HH24:MI:SS';
        if ($values[0] && $values[1]) {
            $values = "TO_DATE( '$values[0] 00:00:00' , '$dateFormat')
                       AND TO_DATE( '$values[1] 23:59:59' , '$dateFormat')";
        } else if ($values[0]) {
            $operator = '>';
            $values = "TO_DATE( '$values[0] 00:00:00' ,'$dateFormat')";
        } else if ($values[1]) {
            $operator = '<';
            $values = "TO_DATE( '$values[1] 00:00:00' ,'$dateFormat')";
        } else {
            $values = "";
        }

        return array($operator, $values);
    }
    private static function buildBetweenCriteria($values) {
        $operator = 'BETWEEN';
        if ($values[0] && $values[1]) {
            $values = "$values[0] AND $values[1]";
        } else if ($values[0]) {
            $operator = '>';
            $values = "$values[0]";
        } else if ($values[1]) {
            $operator = '<';
            $values = "$values[1]";
        } else {
            $values = "";
        }

        return array($operator, $values);

    }

    /**
     * @param $page
     * @param $pageSize
     * @return array
     */
    public static function getPaginationNumber($page, $pageSize)
    {
        $startRowNum = (($page - 1) * $pageSize) + 1;
        $finalRowNum = ($page * $pageSize) + 1;
        return array($startRowNum, $finalRowNum);
    }

    /**
     * When you filters a datasource (like filter actions),
     * this method automates somethings for you.
     * @param $filters
     * @param $tableAlias
     * @return string
     */
    public static function createQueryFiltered($filters, $tableAlias = null)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if($name == 'NRLICENCAUSO'){
                $operator = 'LIKE';
            }
            //(!is_null($values) && $operator == '=') ->Gambi to fix framework bugs :(
            if ((($name != '*' && !empty($values)) || $values == 0) && !(empty($values) && $operator == '=') && !is_null($values)) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode($values, "','") . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE' || $operator == 'LIKE_I') {
                        $operator = 'LIKE';
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode($likes, ' OR ');
                        $filterSQL .= " AND ( $filterLikeAll )";
                    } else if ($operator == 'LIKE') {
                        $filterSQL .= " AND $name $operator $values ";
                    } else if ($operator == 'IS_NULL') {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name IS $values ";
                        } else {
                            $filterSQL .= " AND $name IS $values ";
                        }
                    } else if ($operator == '='){
                        if(!empty($tableAlias)){
                            $filterSQL .= " AND $tableAlias.$name $operator '$values' ";
                        } else {
                            $filterSQL .= " AND $name $operator '$values' ";
                        }
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                $values = "UPPER('" . $mappedLikeAllValues . "')";

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    if (empty($tableAlias)) {
                                        $likes[] = "UPPER($column) LIKE $values";
                                    } else {
                                        $likes[] = "UPPER($tableAlias.$column) LIKE $values";
                                    }
                                }
                                $filterLikeAll = implode($likes, ' OR ');
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name $operator $values ";
                        } else {
                            $filterSQL .= " AND $name $operator $values ";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }
    /**
     * @return EntityManager
     */
    public static function getEntityManager() {
        return InstanceManager::getInstance()->getService('entityManager');
    }

    public static function getCdClienteBase() {
        return InstanceManager::getInstance()->getParameter('cdClienteBase');
    }

    public static function getFileServerUrl() {
        return InstanceManager::getInstance()->getParameter('fileServerUrl');
    }

    public static function getFileServerFilesUrl() {
        return InstanceManager::getInstance()->getParameter('fileServerFilesUrl');
    }

    public static function getClassificationFilter() {
        return InstanceManager::getInstance()->getParameter('classificationFilter');
    }

    public static function currentUserIsTeknisa() {
        $user = self::getEnvironment()->getCurrentUser();
        return self::getCdClienteBase() == $user->getCdcliente();
    }


    public static function validateCurrentClienteUseSLA($cdCliente = null){
        $cdCliente = $cdCliente ? $cdCliente : self::getEnvironment()->getCurrentCdCliente();
        $Validate = Repository::getSlacliente()->findOneBy(array('cdcliente' => $cdCliente));
        return !!$Validate;
    }

    public static function currentClienteIsTeknisa($cdCliente = null) {
        $cdCliente = $cdCliente ? $cdCliente : self::getEnvironment()->getCurrentCdCliente();
        return self::getCdClienteBase() == $cdCliente;
    }

    /**
     * @return Utilities
     */
    public static function getEnvironment(){
        return InstanceManager::getInstance()->getService('\Util\Utilities');
    }

    /**
     * @return \Service\Teknisa\NewCode
     */
    public static function getNewCode(){
        return InstanceManager::getInstance()->getService('\Service\Teknisa\NewCode');
    }
    /**
     * @return \Service\Teknisa\AlphanumericNewCode
     */
    public static function getAlphanumericNewCode(){
        return InstanceManager::getInstance()->getService('\Service\Teknisa\AlphanumericNewCode');
    }

    public static function getCodeByTableName($tableName,$nrorg){
        return self::getNewCode()->getCodeByTableName($tableName,$nrorg);
    }
    public static function getAlphanumericCodeByTableName($tableName,$nrorg){
        return self::getAlphanumericNewCode()->getCodeByTableName($tableName,$nrorg);
    }

    /**
     * @param $entity
     * @return string
     */
    public static function getTableName($entity)
    {
        $entityManager = self::getEntityManager();
        $entityMetadata = $entityManager->getClassMetadata(get_class($entity));
        $tableName = $entityMetadata->getTableName();
        return $tableName;
    }


    public static function parseStringToDate($stringDate)
    {
        if (!empty($stringDate)) {
            $date = new \DateTime($stringDate);
        } else {
            $date = null;
        }
        return $date;
    }

    /**
     * @param array[] $filters
     * @param $nameFilter
     * @param bool $remove
     * @return \DateTime|null
     */
    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    public static function langForIdidioma($lang)
    {
        $ididioma = array(
            'pt' => 'BRA',
            'en' => 'ENG',
            'es' => 'ESP',
        );
        return $ididioma[$lang];
    }


    public static function factoryFromFileData(array $fileData, $destinationFolder,$decode = true) {
        if($decode){
            $filePath = $destinationFolder . DIRECTORY_SEPARATOR . $fileData['name'];
            list($contentType, $base64String) = explode(',', $fileData['b64File']);
            $base64String = base64_decode($base64String);
        }else{
            $filePath = $destinationFolder . DIRECTORY_SEPARATOR . uniqid() . $fileData['name'];
            $contentType = $fileData['type'];
            $base64String = $fileData['file'];
        }
        file_put_contents($filePath, $base64String);
        return new UploadedFile($filePath, $fileData['name'], $contentType, $fileData['size'], null, true);
    }

    public static function getPmStatusByIdAtivo($idativo) {
        $status = 'A';
        if($idativo == 'N'){
            $status = 'I';
        }
        return $status;
    }

    public static function encodeToUtf8($string){
        if(mb_detect_encoding($string) != "UTF-8"){
            return utf8_encode($string);
        }
        return $string;
    }

    /**
     * Converter RTF to text plain replacing unicode characters to ASCII special characteres
     * @param $entity
     * @return string
     */

    public static function rtf2textPlain($text, $ignoreException = true, $useUtf8Encode = false)
    {
        if (!strlen($text) || substr($text,0,5) != '{\rtf') {
            if ($ignoreException){
                $ansa = mb_check_encoding($text, 'UTF-8') ? $text :  utf8_encode($text);
            } else {
                throw new \Exception('Invalid RTF!');
            }
        } else {
            // we'll try to fix up the parts of the rtf as best we can
            // clean up the file a little to simplify parsing
            $text = str_replace("\r",' ',$text); // returns
            $text = str_replace("\n",' ',$text); // new lines
            $text = str_replace('  ',' ',$text); // double spaces
            $text = str_replace('} {','}{',$text); // embedded spaces
            // skip over the heading stuff
            $j = strpos($text,'{',1); // skip ahead to the first part of the header

            $loc = 1;
            $t = "";

            $ansa = "";
            $len = strlen($text);
            self::getpgraph($text, $j, $len); // skip by the first paragrap

            while($j<$len) {
                $c=substr($text,$j,1);
                if ($c=="\\") {
                    // have a tag
                    $tag = self::gettag($text, $j, $len);
                    if (strlen($tag)>0) {
                    // process known tags
                        switch ($tag) {
                            case 'par':
                            $ansa.="\r\n";
                            break;
                            // ad a list of common tags
                            // parameter tags
                            case 'spriority1':
                            case 'fprq2':
                            case 'author':
                            case 'operator':
                            case 'sqformat':
                            case 'company':
                            case 'xmlns1':
                            case 'wgrffmtfilter':
                            case 'pnhang':
                            case 'themedata':
                            case 'colorschememapping':
                                $tt = self::gettag($text, $j, $len);
                                break;
                            case '*':
                            case 'info':
                            case 'stylesheet':
                                // gets to end of paragraph
                                $j--;
                                self::getpgraph($text, $j, $len);
                            default:
                                self::convertTagToSpecialCharacter($tag, $ansa);
                        }
                    }
                } else {
                    $ansa.=$c;
                }
                $j++;
            }
            $ansa = str_replace('{','',$ansa);
            $ansa = str_replace('}','',$ansa);
        }
        return $ansa;
    }

    private static function convertTagToSpecialCharacter($tag, &$ansa){

        $arrayNumber = ['0','1','2','3','4','5','6','7','8','9'];
        $lastOccurrence = 0;

        if (substr($tag, 0, 1) == '\''){

            $tag = str_replace('\\', '', $tag);
            $tag = str_replace('\'', '', $tag);

            //get last occurrence of the number
            foreach ($arrayNumber as $number) {
                $tmpLastPos = strrpos($tag, $number);
                if ($lastOccurrence < $tmpLastPos){
                    $lastOccurrence = $tmpLastPos;
                }
            }

            $tagLength = strlen($tag);

            $c = substr($tag, 0, $lastOccurrence==0 ? 2 : $lastOccurrence+1);
            $unicodeChar = '\u';
            //create unicode char with tag code RTF
            if (strlen($c) == 1){
                $unicodeChar = '\u000'. $c;
            } else if (strlen($c) == 2){
                $unicodeChar = '\u00'. $c;
            } else if (strlen($c) == 3){
                $unicodeChar = '\u0'. $c;
            } else if (strlen($c) == 4){
                $unicodeChar = '\u'. $c;
            }

            //decode unicode to ascii
            $ansa .= json_decode('"'.$unicodeChar.'"');


            //get characters after tag code rtf
            if ($tagLength > $lastOccurrence+1){
                $lenghtToGet = $tagLength-($lastOccurrence+1);
                $ansa.= substr($tag, $lastOccurrence+1, $lenghtToGet) ." ";
            }
        }
    }


    private static function gettag(&$text, &$j, &$len) {
        // gets the text following the / character or gets the param if it there

        $tag='';
        while(true) {
            $j++;
            if ($j>=$len) break;
            $c = substr($text,$j,1);
            if ($c==' ') break;
            if ($c==';') break;
            if ($c=='}') break;
            if ($c=="\\") {
                $j--;
                break;
            }
            if ($c=="{") {
                break;
            }
            if ((($c>='0')&&($c<='9'))||(($c>='a')&&($c<='z'))||(($c>='A')&&($c<='Z'))||$c=="'"||$c=="-"||$c=="*" ){
                $tag=$tag.$c;
            } else {
                $j--;
                break;
            }
        }
        return $tag;
    }

    private static function getpgraph(&$text, &$j, &$len) {
        // if the first char after a tag is { then throw out the entire paragraph
        // this has to be nested
        $nest=0;
        while(true) {
            $j++;
            if ($j>=$len) break;
            if (substr($text,$j,1)=='}') {
                if ($nest==0) return;
                $nest--;
            }
            if (substr($text,$j,1)=='{') {
                $nest++;
            }
        }
    }

    public static function executeFunction($object,$arrayMethod){
        foreach ($arrayMethod as $method) {
            $object = $object->$method();
        }
        return $object;
    }

    public static function getArrayByArrayObjectWithGet( array $arrayObject = array(),$arrayMethod = array('getId')){
        $array = array();
        foreach ($arrayObject as $object) {
            $array[] = self::executeFunction($object,$arrayMethod);
        }
        return $array;
    }

    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }

    public static function downloadFile($fileDir, $fileName, $fileType = 'text/plain', $fileSize = null) {
        if(!$fileSize) {
            $fileSize = filesize($fileSize);
        }
        header('Accept-Ranges: none');
        header("Content-Type: " . $fileType);
        header("Content-Length: " . $fileSize);
        header('Content-Disposition: attachment; filename="' . $fileName.'"');

        readfile($fileDir);
        exit;
    }

    /**
     * @param CustomException $e
     * @return Error
     */
    public static function getErrorObjectFromException(CustomException $e) {
        $error = array (
            'CODE'   => $e->getMessage(),
            'PARAMS' => $e->getParams()
        );
        return new Error(json_encode($error), $e->getCode(), $e->getTraceAsString());
    }

    public static function getOptionsSoap() {
        ini_set('default_socket_timeout', 7200);
        return array(
            'encoding'    => 'UTF-8',
            'trace'       => true,
            'exceptions'  => true,
            'connection_timeout' => 7200,
            'keep_alive' => false,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
            'stream_context' => stream_context_create(array(
                'ssl' => array(
                    'verify_peer' => false,
                    'verify_peer_name' => false,
                    'allow_self_signed' => true
                )
            ))

        );
    }

    public function convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName)
    {
        $zmartFilterCriteria = new ZmartFilterCriteria($dataSourceName);
        foreach($conditions as $condition) {
            $name = isset($condition['name']) ? $condition['name'] : $condition['columnName'];
            switch (true) {
                case $this->isGroupBy($condition):
                    $zmartFilterCriteria->setGroupBy($name);
                    break;
                case $this->isAggregateFunction($condition):
                    $zmartFilterCriteria->addAggregateFunction($this->getLabelOrName($condition), $condition['operator'], $name);
                    break;
                case $this->isOrderBy($condition):
                    $zmartFilterCriteria->addOrderBy($name['name'], $name['direction']);
                    break;
                default:
                    $zmartFilterCriteria->addCondition($name, $condition['operator'], $condition['value']);
            }
        }
        return $zmartFilterCriteria;
    }

    protected function isGroupBy($condition) {
        return $condition['operator'] === ZmartFilterCriteria::GROUP_BY;
    }

    protected function isAggregateFunction($condition) {
        $functions = array(
            ZmartFilterCriteria::SUM,
            ZmartFilterCriteria::AVG,
            ZmartFilterCriteria::MIN,
            ZmartFilterCriteria::MAX,
            ZmartFilterCriteria::COUNT
        );
        return in_array($condition['operator'], $functions);
    }

    protected function isOrderBy($condition) {
        return $condition['operator'] === ZmartFilterCriteria::ORDER_BY;
    }

    public function getMongoDate($date, $format = 'd/m/Y')
    {
        if(!($date instanceof \DateTime)) {
            $timezone = new \DateTimeZone('UTC');
            $date = \DateTime::createFromFormat($format, $date, $timezone);
        }
        return new MongoDate($date->getTimestamp() * 1000);
    }
    const IV_SIZE = 16;
    const TEXT_SIZE = 32;


    /**
     * Pad the text
     *
     * Pad the text based on the normalization of string size to be encrypted
     *
     * @param $salt
     * @return string
     */
    protected static function padText($text, $size) {
        if (strlen($text) % $size)
            return str_pad($text, strlen($text) + $size - strlen($text) % $size, "\0");
        return $text;
    }

    protected static function getIv($cipher){
        return openssl_random_pseudo_bytes(openssl_cipher_iv_length($cipher));
    }

    private static function encodeEncryptedText($text, $iv){
        $iv_pos = rand(1, 15);
        $text = substr_replace($text, $iv, $iv_pos, 0);
        $iv_pos = dechex($iv_pos);
        $text = substr_replace($text, $iv_pos, 0, 0);
        $text = base64_encode($text);

        return $text;
    }

    /**
     * Encrypt data with salt.
     *
     * @param string $text Value to be encrypt.
     * @param string $salt Salt to be encrypt.
     *
     * @return string
     */
    public static function encrypt($text, $salt = 'libraries')
    {
        try{
            $cipher = "AES-256-CBC";
            $iv = self::getIv($cipher);
            $text = self::padText($text, 32);
            $text =
                openssl_encrypt(
                    $text,
                    $cipher,
                    $salt,
                    OPENSSL_NO_PADDING,
                    $iv
                );
            $text = self::encodeEncryptedText($text, $iv);

            return $text;
        }
        catch(\Exception $e){
            throw $e::internalError();
        }
    }

    private static function decodeEncryptedText($text){
        $text = base64_decode($text);
        $iv_pos = hexdec($text[0])+1;
        $iv = substr($text, $iv_pos, self::IV_SIZE);
        $text_arr = explode($iv, $text);
        $text_arr[0] = substr($text_arr[0], 1);
        $text = implode('', $text_arr);
        $iv = self::padText($iv, self::IV_SIZE);

        return array('text'=> $text, 'iv'=>$iv);
    }

    /**
     * Decrypt data through salt.
     *
     * @param string $text Value to be decrypt.
     * @param string $salt Salt used in encryption.
     *
     * @return string
     */
    public static function decrypt($text, $salt = 'libraries')
    {
        try {
            $cipher = "AES-256-CBC";
            $decoded = self::decodeEncryptedText($text);

            $text =
                openssl_decrypt(
                    $decoded['text'],
                    $cipher,
                    $salt,
                    OPENSSL_NO_PADDING,
                    $decoded['iv']
                );
            $text = trim($text);
            return $text;
        }
        catch(\Exception $e){
            throw $e::internalError();
        }
    }


    public static function downloadAndRemoveFile($fileDir, $fileName, $fileType = 'text/plain', $isRemoveFile = true) {
        if(strpos($fileDir, 'http') === false) {
            $fileSize = filesize($fileDir);
        } else {
            $fileSize = self::getRemoteFileSize($fileDir);
        }
        header('Accept-Ranges: none');
        header("Content-Type: " . $fileType . "; charset=ISO-8859-1");
        header("Content-Length: " . $fileSize);
        header("Content-Disposition: attachment; filename=" . $fileName);
        readfile($fileDir);

        if($isRemoveFile) {
            unlink($fileDir);
        }
        exit;
    }

    public static function getRemoteFileSize($url) {
        $data = get_headers($url, true);
        if (isset($data['Content-Length'])) {
            return (int)$data['Content-Length'];
        }
        return null;
    }
}
